'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _parseFloat = require('babel-runtime/core-js/number/parse-float');

var _parseFloat2 = _interopRequireDefault(_parseFloat);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.checkDateValid = checkDateValid;
exports.checkReg = checkReg;
exports.checkValueType = checkValueType;
exports.beingNil = beingNil;
exports.tofixed = tofixed;
exports.SimpleDateFormat = SimpleDateFormat;
exports.checkParamCount = checkParamCount;
exports.reverse = reverse;
exports.removeNullParam = removeNullParam;
exports.replaceNullParam = replaceNullParam;
exports.getFirstNum = getFirstNum;

var _util = require('util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var moment = require('moment');
var FormulaError = require('./FormulaError').FormulaError;

/**
 * 检查值的是否为有效的日期，并抛出错误
 * @param {*} value 要检查类型的值
 * @param {*} funcName 函数名称，用于类型检查报错
 * @param {*} index 被校验参数的位置
 */
function checkDateValid(value, funcName, index) {
  try {
    if (!(0, _util.isNullOrUndefined)(value) && value.length !== 0) {
      if (!Array.isArray(value)) {
        if (!moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss'], true).isValid()) {
          throw new Error(index);
        }
      } else {
        value.forEach(function (e, i) {
          if (!moment(e, 'YYYY-MM-DD', true).isValid()) {
            throw new Error(index + i);
          }
        });
      }
    }
  } catch (e) {
    var err = new FormulaError({
      errCode: 1,
      funcName: funcName,
      paramIdx: e.message
    });
    throw err;
  }
}

/**
 * 检查值的指定校验格式，并抛出错误
 * @param {*} value 要检查类型的值
 * @param {*} reg 正则格式
 * @param {*} funcName 函数名称，用于类型检查报错
 * @param {*} index 被校验参数的位置
 */
function checkReg(value, reg, funcName, index) {
  var pattern = new RegExp(reg);
  if (!pattern.test(value)) {
    var err = new FormulaError({
      errCode: 1,
      funcName: funcName,
      paramIdx: index
    });
    throw err;
  }
}

/**
 * 检查值的类型，并抛出错误
 * @param {*} value 要检查类型的值
 * @param {*} type 值应该的类型，如“number”，“boolean”， “string”，必须字符串
 * @param {*} funcName 函数名称，用于类型检查报错
 * @param {*} startIdx 参数开始开始的位置，从0开始
 */
function checkValueType(type, funcName, startIdx) {
  for (var _len = arguments.length, values = Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
    values[_key - 3] = arguments[_key];
  }

  var _ref;

  values = (_ref = []).concat.apply(_ref, (0, _toConsumableArray3.default)(values));
  values.forEach(function (value, index) {
    if (!(0, _util.isNullOrUndefined)(value) && (typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value)) !== type) {
      var err = new FormulaError({
        errCode: 1,
        funcName: funcName,
        paramIdx: startIdx + index + 1
      });
      throw err;
    }
  });
}

function beingNil(value) {
  return typeof value === 'undefined' || value === null;
}

/** 保留小数 解决 Number.toFixed 精度问题 */
function tofixed(num, n) {
  var symbol = 1;
  // 符号为负
  if (num < 0) {
    symbol = -1;
    num *= -1;
  }
  var num2 = (Math.round(num * Math.pow(10, n)) / Math.pow(10, n) + Math.pow(10, -(n + 1))).toString().slice(0, -1);
  return parseFloat(num2 * symbol).toFixed(n);
}

/**
 * 时间格式化
 */
function SimpleDateFormat(pattern) {
  var fmt = new Object();
  fmt.pattern = pattern;

  fmt.parse = function (source) {
    try {
      if (!/\d{4}(\-|\/|.)\d{1,2}\1\d{1,2}/.test(source)) {
        throw 'invalid';
      }
      var date = new Date(source.replace(/-/g, "/"));
      if (source.length <= 10) {
        date.setHours(0);
      }
      if (date.toString() === 'Invalid Date' || (0, _util.isNullOrUndefined)(date) || isNaN(date) || date === '') {
        throw 'invalid';
      }
      return date;
    } catch (e) {
      return 'Invalid Value';
    }
  };

  fmt.format = function (date) {
    try {
      if (!/^[0-9]*$/.test(date)) {
        throw 'invalid';
      }
      if (date.toString().length !== 10 && date.toString().length !== 13) {
        throw 'invalid';
      }
      if (typeof date === 'string') {
        date = parseInt(date, 10);
      }
      if ((0, _util.isNullOrUndefined)(date) || date === '' || isNaN(date)) {
        throw 'invalid';
      }
      date = new Date(date);
    } catch (e) {
      return 'Invalid Value';
    }

    var strTime = this.pattern; //时间表达式的正则
    var o = {
      'M+': date.getMonth() + 1, //月份
      'd+': date.getDate(), //日
      'H+': date.getHours(), //小时
      'm+': date.getMinutes(), //分
      's+': date.getSeconds(), //秒
      'q+': Math.floor((date.getMonth() + 3) / 3), //季度
      S: date.getMilliseconds() //毫秒
    };
    if (/(y+)/.test(strTime)) {
      strTime = strTime.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
    }
    for (var k in o) {
      if (new RegExp('(' + k + ')').test(strTime)) {
        strTime = strTime.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ('00' + o[k]).substr(('' + o[k]).length));
      }
    }
    return strTime;
  };
  return fmt;
}

/**
 * 检查参数的个数是否正确
 * @param {*} funcName 方法名称
 * @param {*} count 期待参数个数
 * @param {*} params 参数列表，即arguments参数
 */
function checkParamCount(funcName, count, params) {
  if (count !== params.length) {
    var err = new FormulaError({
      errCode: 2,
      funcName: funcName,
      expectNum: count,
      actualNum: params.length
    });
    throw err;
  }
}

// 字符串反转
function reverse(value) {
  return String(value).split('').reverse().join('');
}

/**
 * 去除数组中为null或者undefined的值
 * @param {*} values 需要剔除无用数值的数组
 */
function removeNullParam(values) {
  var resultValues = [];
  for (var i = 0; i < values.length; i++) {
    if (!(0, _util.isNullOrUndefined)(values[i])) {
      resultValues.push(values[i]);
    }
  }
  return resultValues;
}

/**
 * 数组中null或者undefined的值替换为0
 * @param {*} values
 */
function replaceNullParam(values) {
  var resultValues = [];
  for (var i = 0; i < values.length; i++) {
    var tempVal = values[i];
    if ((0, _util.isNullOrUndefined)(tempVal) || isNaN(tempVal)) {
      tempVal = 0;
    }
    resultValues.push(tempVal);
  }
  return resultValues;
}

/**
 * 获取字符串中第一个数字，小数也可以获取
 */
function getFirstNum(val) {
  if ((0, _util.isNullOrUndefined)(val)) {
    return null;
  }
  val = val.toString();
  var matches = val.match(/(-)*[0-9]\d*(\.\d+)?/g);
  if ((0, _util.isNullOrUndefined)(matches) || matches.length < 1) {
    return null;
  } else {
    return (0, _parseFloat2.default)(matches[0]);
  }
}

// 加减乘除精度运算
Number.prototype.add = function (arg) {
  var r1 = 0;
  var r2 = 0;
  var m = 0;
  if (this.toString().indexOf('.') !== -1) {
    try {
      r1 = this.toString().split('.')[1].length;
    } catch (e) {}
  }
  if (arg.toString().indexOf('.') !== -1) {
    try {
      r2 = arg.toString().split('.')[1].length;
    } catch (e) {}
  }
  m = Math.pow(10, Math.max(r1, r2));
  return (this.mul(m) + arg.mul(m)).div(m);
};

Number.prototype.sub = function (arg) {
  return this.add(-arg);
};

Number.prototype.mul = function (arg) {
  var m = 0;
  var s1 = this.toString();
  var s2 = arg.toString();
  if (s1.indexOf('.') !== -1) {
    try {
      m += s1.split('.')[1].length;
    } catch (e) {}
  }
  if (s2.indexOf('.') !== -1) {
    try {
      m += s2.split('.')[1].length;
    } catch (e) {}
  }
  return Number(s1.replace('.', '')) * Number(s2.replace('.', '')) / Math.pow(10, m);
};

Number.prototype.div = function (arg) {
  var r1 = 0;
  var r2 = 0;
  if (this.toString().indexOf('.') !== -1) {
    try {
      r1 = this.toString().split('.')[1].length;
    } catch (e) {}
  }
  if (arg.toString().indexOf('.') !== -1) {
    try {
      r2 = arg.toString().split('.')[1].length;
    } catch (e) {}
  }
  var m = Math.pow(10, Math.max(r1, r2));
  var thisMinus = this < 0 ? -1 : 1;
  var argMinus = arg < 0 ? -1 : 1;
  // 加个0.5，然后再取整，以保障整数的准确
  var a = parseInt(Math.abs(this) * m + 0.5) * thisMinus;
  var b = parseInt(Math.abs(arg) * m + 0.5) * argMinus;
  return a / b;
};

Number.prototype.decimalCnt = function () {
  if (this.toString().indexOf('.') === -1) {
    return 0;
  }
  return this.toString().split('.')[1].length;
};