import { isNullOrUndefined } from 'util';
var moment = require('moment');
var FormulaError = require('./FormulaError').FormulaError;

/**
 * 检查值的是否为有效的日期，并抛出错误
 * @param {*} value 要检查类型的值
 * @param {*} funcName 函数名称，用于类型检查报错
 * @param {*} index 被校验参数的位置
 */
export function checkDateValid(value, funcName, index) {
  try {
    if (!isNullOrUndefined(value) && value.length !== 0) {
      if (!Array.isArray(value)) {
        if (!moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss'], true).isValid()) {
          throw new Error(index);
        }
      } else {
        value.forEach((e, i) => {
          if (!moment(e, 'YYYY-MM-DD', true).isValid()) {
            throw new Error(index + i);
          }
        })
      }
    }
  } catch (e) {
    let err = new FormulaError({
      errCode: 1,
      funcName: funcName,
      paramIdx: e.message
    });
    throw err;
  }
}

/**
 * 检查值的指定校验格式，并抛出错误
 * @param {*} value 要检查类型的值
 * @param {*} reg 正则格式
 * @param {*} funcName 函数名称，用于类型检查报错
 * @param {*} index 被校验参数的位置
 */
export function checkReg(value, reg, funcName, index) {
  var pattern =new RegExp(reg);
  if (!pattern.test(value)) {
    let err = new FormulaError({
      errCode: 1,
      funcName: funcName,
      paramIdx: index
    });
    throw err;
  }
}

/**
 * 检查值的类型，并抛出错误
 * @param {*} value 要检查类型的值
 * @param {*} type 值应该的类型，如“number”，“boolean”， “string”，必须字符串
 * @param {*} funcName 函数名称，用于类型检查报错
 * @param {*} startIdx 参数开始开始的位置，从0开始
 */
export function checkValueType(type, funcName, startIdx, ...values) {
  values = [].concat(...values);
  values.forEach((value, index) => {
    if (!isNullOrUndefined(value) && typeof value !== type) {
      let err = new FormulaError({
        errCode: 1,
        funcName: funcName,
        paramIdx: startIdx + index + 1
      });
      throw err;
    }
  })
}

export function beingNil(value) {
  return typeof value === 'undefined' || value === null;
}

/** 保留小数 解决 Number.toFixed 精度问题 */
export function tofixed (num, n) {
  var symbol = 1;
  // 符号为负
  if (num < 0) {
    symbol = -1;
    num *= -1;
  }
  var num2 = (Math.round(num * Math.pow(10, n)) / Math.pow(10, n) + Math.pow(10, -(n + 1))).toString().slice(0, -1);
  return parseFloat(num2 * symbol).toFixed(n);
}

/**
 * 时间格式化
 */
export function SimpleDateFormat(pattern) {
  var fmt = new Object();
  fmt.pattern = pattern;

  fmt.parse = function (source) {
    try {
      if (!/\d{4}(\-|\/|.)\d{1,2}\1\d{1,2}/.test(source)) {
        throw 'invalid';
      }
      const date = new Date(source.replace(/-/g, "/"));
      if (source.length <= 10) {
        date.setHours(0)
      }
      if (date.toString() === 'Invalid Date' || isNullOrUndefined(date) || isNaN(date) || date === '') {
        throw 'invalid';
      }
      return date;
    } catch (e) {
      return 'Invalid Value';
    }
  }

  fmt.format = function (date) {
    try {
      if (!/^[0-9]*$/.test(date)) {
        throw 'invalid';
      }
      if (date.toString().length !== 10 && date.toString().length !== 13) {
        throw 'invalid';
      }
      if (typeof date === 'string') {
        date = parseInt(date, 10);
      }
      if (isNullOrUndefined(date) || date === '' || isNaN(date)) {
        throw 'invalid';
      }
      date = new Date(date);
    } catch (e) {
      return 'Invalid Value';
    }

    var strTime = this.pattern; //时间表达式的正则
    var o = {
      'M+': date.getMonth() + 1, //月份
      'd+': date.getDate(), //日
      'H+': date.getHours(), //小时
      'm+': date.getMinutes(), //分
      's+': date.getSeconds(), //秒
      'q+': Math.floor((date.getMonth() + 3) / 3), //季度
      S: date.getMilliseconds(), //毫秒
    };
    if (/(y+)/.test(strTime)) {
      strTime = strTime.replace(
        RegExp.$1,
        (date.getFullYear() + '').substr(4 - RegExp.$1.length)
      )
    }
    for (var k in o) {
      if (new RegExp('(' + k + ')').test(strTime)) {
        strTime = strTime.replace(
          RegExp.$1,
          RegExp.$1.length == 1
            ? o[k]
            : ('00' + o[k]).substr(('' + o[k]).length)
        )
      }
    }
    return strTime;
  }
  return fmt;
}

/**
 * 检查参数的个数是否正确
 * @param {*} funcName 方法名称
 * @param {*} count 期待参数个数
 * @param {*} params 参数列表，即arguments参数
 */
export function checkParamCount(funcName, count, params) {
  if (count !== params.length) {
    let err = new FormulaError({
      errCode: 2,
      funcName: funcName,
      expectNum: count,
      actualNum: params.length
    });
    throw err;
  }
}

// 字符串反转
export function reverse(value) {
  return String(value).split('').reverse().join('');
}

/**
 * 去除数组中为null或者undefined的值
 * @param {*} values 需要剔除无用数值的数组
 */
export function removeNullParam(values) {
  let resultValues = [];
  for (let i = 0; i < values.length; i++) {
    if (!isNullOrUndefined(values[i])) {
      resultValues.push(values[i]);
    }
  }
  return resultValues;
}

/**
 * 数组中null或者undefined的值替换为0
 * @param {*} values
 */
export function replaceNullParam(values) {
  let resultValues = [];
  for (let i = 0; i < values.length; i++) {
    let tempVal = values[i];
    if (isNullOrUndefined(tempVal) || isNaN(tempVal)) {
      tempVal = 0;
    }
    resultValues.push(tempVal);
  }
  return resultValues;
}

/**
 * 获取字符串中第一个数字，小数也可以获取
 */
export function getFirstNum(val) {
  if (isNullOrUndefined(val)) {
    return null;
  }
  val = val.toString()
  let matches = val.match(/(-)*[0-9]\d*(\.\d+)?/g)
  if (isNullOrUndefined(matches) || matches.length < 1) {
    return null;
  } else {
    return Number.parseFloat(matches[0]);
  }
}

// 加减乘除精度运算
Number.prototype.add = function (arg) {
  let r1 = 0;
  let r2 = 0;
  let m = 0;
  if (this.toString().indexOf('.') !== -1) {
    try {
      r1 = this.toString().split('.')[1].length;
    } catch (e) {}
  }
  if (arg.toString().indexOf('.') !== -1) {
    try {
      r2 = arg.toString().split('.')[1].length;
    } catch (e) {}
  }
  m = Math.pow(10, Math.max(r1, r2));
  return (this.mul(m) + arg.mul(m)).div(m);
}

Number.prototype.sub = function (arg) {
  return this.add(-arg);
}

Number.prototype.mul = function (arg) {
  let m = 0;
  let s1 = this.toString();
  let s2 = arg.toString();
  if (s1.indexOf('.') !== -1) {
    try {
      m += s1.split('.')[1].length;
    } catch (e) {}
  }
  if (s2.indexOf('.') !== -1) {
    try {
      m += s2.split('.')[1].length;
    } catch (e) {}
  }
  return (
    (Number(s1.replace('.', '')) * Number(s2.replace('.', ''))) /
    Math.pow(10, m)
  )
}

Number.prototype.div = function (arg) {
  let r1 = 0;
  let r2 = 0;
  if (this.toString().indexOf('.') !== -1) {
    try {
      r1 = this.toString().split('.')[1].length;
    } catch (e) {}
  }
  if (arg.toString().indexOf('.') !== -1) {
    try {
      r2 = arg.toString().split('.')[1].length;
    } catch (e) {}
  }
  const m = Math.pow(10, Math.max(r1, r2));
  const thisMinus = this < 0 ? -1 : 1;
  const argMinus = arg < 0 ? -1 : 1;
  // 加个0.5，然后再取整，以保障整数的准确
  const a = parseInt(Math.abs(this) * m + 0.5) * thisMinus;
  const b = parseInt(Math.abs(arg) * m + 0.5) * argMinus;
  return a/b;
}

Number.prototype.decimalCnt = function () {
  if (this.toString().indexOf('.') === -1) {
    return 0;
  }
  return this.toString().split('.')[1].length;
}
